****f* SuSeFLAV/softspectrum.f 
*  NAME
*    softspectrum
*  SYNOPSIS
*    Subroutine to generate Susy spectrum. 
*  FUNCTION
*     THIS PROGRAM CALCULATES THE MASSES AND MIXINGS IN THE SOFT
*     SECTOR FOR A GIVEN SPARTICLE MASSES AT LOW ENERGIES. THIS
*     PARAMETERS CAN BE EITHER OUTPUT FROM RGE OR DIRECT INPUTS.
*     AT PRESENT, IT HAS MAINLY SLEPTONIC AND SNEUTRINO PART ALONG
*     WITH CHARGINO AND NEUTRALINO PARTS IN THE BASIS WHERE CHARGED
*     LEPTONS ARE CONSIDERED DIAGONAL.
*     Higgs Spectrum added. Last Modified : 26/01/10. 
*
*  INPUTS
*     tanbeta  = value of the tanbeta being used. 
*     mSLRG    = (3 X 3) real mass matrix of left-handed sleptons (L)
*     mSERG    = (3 X 3) real mass matrix of right-handed sleptons (E^c)
*     AERG     = (3 X 3) real mass matrix of leptonic A-parameters.
*     M1tz     = low-energy (~M_Z) value of parameter M1
*     M2tz     = low-energy (~M_Z) value of parameter M2
*     mur      =  low-energy value of the mu-parameter either through REWSB
*            or as a free parameter.
*
*  RESULT
*     SUegg = 6 eigenvalues (ascending order) of UP-Squark mass matrix.
*     USU = (6 X 6) real orthogonal matrix such that,
*            USU*MSU*Transpose[USU] = Diag[MSU].
*     SUeg = 6 singular values (descending order) of UP-Squark mass
*            mass matrix. All positive.
*
*     SDegg = 6 eigenvalues (ascending order) of Down-Squark mass matrix.
*     USD = (6 X 6) real orthogonal matrix such that,
*            USD*MSD*Transpose[USD] = Diag[MSD].
*     SDeg = 6 singular values (descending order) of Down-Squark mass
*            mass matrix. All positive.
*
*     Slegg = 6 eigenvalues (ascending order) of slepton mass matrix.
*     USL = (6 X 6) real orthogonal matrix such that,
*            USL*MSL*Transpose[USL] = Diag[MSL].
*     SLeg = 6 singular values (descending order) of slepton mass
*            mass matrix. All positive.
*	
*     SNegg = 3 eigenvalues (ascending order) of sneutrino mass matrix.
*     USN = (3 X 3) real orthogonal matrix such that
*             USN*mSN*Transpose[USN] = Diag[MSN].
*     SNeg = 3 singular values (descending order) of sneutrino mass matrix.
*
*     ON = (4 X 4) orthogonal matrix such that 
*           ON.MNeut.Transpose[ON] = Diag[MNeut] 
*     Neg = 4 singular values (descending order) of the Neutralino mass matrix. 
*
*     OCR, OCL = (2 X 2) orthogonal matrices such that 
*                MChar = Transpose[OCR].Diag[MChar].OCL
*     Ceg = 2 singular values of the Neutralino Mass Matrix
*
*     AOK = Tells us whether all the diagonalising routines have run alright
*           or not. Without the Higgs spectrum, AOK should be 10 on output, 
*	    if everything goes well. 
*
*  EXAMPLE
*      subroutine softspectrum(tanbeta,mSQRG,mSDRG,mSURG,AURG,ADRG,vev1,
*     $     vev2,mSLRG,mSERG,AERG,yuRG,yeRG,ydRG,M1tz,M2tz,mur,SUegg,USU,
*     $     SUeg,SDegg,USD,SDeg,SLegg,USL,SLeg,SNegg,USN,SNeg,ON,Neg,OCR,
*     $     OCL,Ceg,AOK,MT_susy,MB_susy,Mtau_susy,mh0sq,
*     $     mhu0sq,mhpmsq,mA0sq,Neuevi,ONL,ONR)
*  
*  NOTES
*     ALL Masses and Parameters are in GeV.
*     Lapack is required.
*     Note that Lapack returns singular values as A = U Diag [A] Transpose[V]
*     or Transpose[U] A V = Diag[A]  which is transpose of the conventions of 
*     Mathematica as well as our conventions. (See notes). Our conventions
*     coincide with that of HN.  Also compare with that of Haber-Kane.  
*     
*  BUGS
*     
*     Possibly there are no bugs in this subroutine.
*     
*  SEE ALSO
*     -----
******
* You can use this space for remarks that should not be included
* in the documentation.
C****C
!===========================================================================================
C\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
C     1. Checked on 28/06/2010
C     2. Removed the unused variables.
C     3. Removed the dependency on 'stddef.dat'.
C\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\


      subroutine softspectrum(scale,sgnmu,tanbeta,mSQRG,mSDRG,mSURG,
     $     AURG,ADRG,vev1,vev2,MWsq, MZsq,
     $     mSLRG,mSERG,AERG,yuRG,yeRG,ydRG,M1tz,M2tz,mur,bmur,
     $     SUegg,USU,
     $     SUeg,SDegg,USD,SDeg,SLegg,USL,SLeg,SNegg,USN,SNeg,MNeut,ON,
     $     Neg,MChar,mSL2,OCR,OCL,Ceg,AOK,MT_susy,MB_susy,Mtau_susy,
     $     mh0sq,mhu0sq,mhpmsq,mA0sq,Neuevi,ONL,ONR)


      implicit none 

      integer m,a,i,j
      integer AOK,lwork1,lwork2,lwork3,info,lwork
      character*10 flag_bmu

      parameter(lwork=35,lwork1=15,lwork2=20,lwork3=10)

      double precision mSL(6,6),mSL2(6,6),mSLRG(3,3),AERG(3,3)
      double precision MChar(2,2),tanbeta, mur, M1tz, SLeg(6), SLegi(6)
      double precision MNeut(4,4),M2tz,mSN(3,3),USL(6,6),mSERG(3,3)
      double precision USN(3,3),SNeg(3),Neg(4),ON(4,4), OCRT(2,2)
      double precision work(lwork), USNT(3,3), OCR(2,2), OCL(2,2)
      double precision OCLT(2,2),mSL1(6,6),Ceg(2),mSN1(3,3)
      double precision work1(lwork1),USLT(6,6),work2(lwork2)
      double precision sw2,stw,ctw,beta,sbeta,cbeta,work3(lwork3)
      double precision vev1,vev2,cos2beta,SLegg(6),SNegg(3),mSU2(6,6)
      double precision mSU(6,6),mSQRG(3,3),mSURG(3,3),AURG(3,3)
      double precision mSU1(6,6),USU(6,6),USUT(6,6),SUegg(6),SUeg(6)
      double precision mSD(6,6),mSDRG(3,3),ADRG(3,3),mSD1(6,6),mSD2(6,6)
      double precision USD(6,6),USDT(6,6),SDegg(6),SDeg(6),slegs(6)

      double precision AURGn(3,3),ADRGn(3,3),AERGn(3,3)
      double precision MWsq,MZsq,scale,mSD3(6,6)
      double precision mh0sq,mhu0sq,mhpmsq,mA0sq
      DOUBLE PRECISION Mhiggstree(2,2), Mhiggsn(2,2), alphatree
      DOUBLE PRECISION heign(2), lhim(2),hl(2,2),hr(2,2)



      double precision yuRG(3,3),ydRG(3,3),yeRG(3,3)
      double precision mT, mB, mTau !,SUeggt(6)
      double precision MNeut1(4,4),MChar1(2,2),bmur
      double precision MNeut2(4,4),Neuevi(4),ONL(4,4),ONR(4,4)

      double precision SUeggi(6)
      
      DOUBLE PRECISION sups(2,2), schs(2,2), stops(2,2),supegg(2),
     $     schegg(2), stopegg(2)

      DOUBLE PRECISION sdowns(2,2),sstrs(2,2), sbtms(2,2),sdownegg(2),
     $     sstregg(2), sbtmegg(2)

      DOUBLE PRECISION staus(2,2),smus(2,2), sels(2,2),staugg(2),
     $     smugg(2), selgg(2)


      DOUBLE PRECISION  thetaocl

      double precision  Mceg(2,2), thetaocr, OCRdum(2,2),OCLdum(2,2)
      double precision mmdag(2,2), mdagm(2,2), MCharT(2,2)
      DOUBLE PRECISION eigcgR(2,2),eigcgL(2,2)
      
      DOUBLE PRECISION supsi(2,2), supL(2,2), supR(2,2)
      DOUBLE PRECISION schsi(2,2), schL(2,2), schR(2,2)
      DOUBLE PRECISION stopsi(2,2), stopL(2,2), stopR(2,2)
      
      DOUBLE PRECISION sdownsi(2,2), sdownL(2,2), sdownR(2,2)
      DOUBLE PRECISION sstrsi(2,2), sstrL(2,2), sstrR(2,2)
      DOUBLE PRECISION sbtmsi(2,2), sbtmL(2,2), sbtmR(2,2)

      DOUBLE PRECISION selsi(2,2), selL(2,2), selR(2,2)
      DOUBLE PRECISION smusi(2,2), smuL(2,2), smuR(2,2)
      DOUBLE PRECISION stausi(2,2), stauL(2,2), stauR(2,2)

      DOUBLE PRECISION rrstaumu(2)
            
      DOUBLE PRECISION USLs(6,6),USLTs(6,6),MSL3(6,6)
!     $     USLsrt(6,6),USLTsrt(6,6)
           
      double precision sgnmu
      
      double precision slpmix(6,6),slpmixT(6,6),slpegn(6)
      double precision SDegi(6),sdmix(6,6),sdmixT(6,6),sdegn(6)
      double precision sumix(6,6),sumixT(6,6),suegn(6)
!     double precision ONT(4,4),SUeggi(6),work2(lwork2)
      
      integer ILO,IHI,IWORK
      double precision SCALE1(6),ABNRM, RCONDE(6),RCONDV(6)

!     MT at the susy scale from the running
      double precision MT_susy, MB_susy, Mtau_susy
!      common/suptree/ SUeggt
      common/higgsmixmz/ alphatree 
!      common/mutaup/rrstaumu,USLsrt,USLTsrt
      external dsyev, dgesvd , matmult2d, mat3prod2d,dgeevx
      common/mAflag/flag_bmu

!--------mixing angles 2x2

      DOUBLE PRECISION thetat,thetab,thetatau
      DOUBLE PRECISION thetac,thetas,thetamu
      DOUBLE PRECISION thetau,thetad,thetae

      DOUBLE PRECISION thetatz,thetabz,thetatauz
      DOUBLE PRECISION thetacz,thetasz,thetamuz
      DOUBLE PRECISION thetauz,thetadz,thetaez

      double precision SLegvd(6),SUegvd(6),SDegvd(6)
      double precision USLvd(6,6),USUvd(6,6),USDvd(6,6)

      double precision traceb, detb,eigb1,eigb2
      double precision traceL,DETL,eigl1,eigl2
      double precision traceT,DETT,eigt1,eigt2
 

      common/lfvmixing/USLvd,USUvd,USDvd
      common/lfveigval/SLegvd,SUegvd,SDegvd


      common/sfmixing_susy/thetat,thetab,thetatau,thetac,thetas,thetamu,
     $     thetau,thetad,thetae

      common/sfmixing_mz/thetatz,thetabz,thetatauz,thetacz,thetasz,
     $     thetamuz,thetauz,thetadz,thetaez



C     --------------------------------------------------------------
C     Standard Inputs  and continuation of declaration statements
C     ---------------------------------------------------------------
      
      include 'stdinputs.h'
!------------------------------
      
!     redefining MT
      MT = MT_susy
      MB = MB_susy
      mTau = Mtau_susy
      
C     To be always be included ONLY after the defintion of tanbeta.
C     Here tanbeta is an input. 
      

      MW = dsqrt(MWsq)
      MZ = dsqrt(MZsq)

      
      sw2 = (1.d0 - (MWsq/MZsq))      
      stw = dsqrt(sw2)
      ctw = dsqrt(1.d0 - sw2)
      sinsqtw = sw2
      
      beta  = datan(tanbeta)
      sbeta = dsin(beta)
      cbeta = dcos(beta)
      cos2beta = cbeta*cbeta - sbeta*sbeta


      loopmulin: do i = 1, 3
      loopmuljn: do j = 1, 3

      AURGn(i,j) = AURG(i,j)
      
      ADRGn(i,j) = ADRG(i,j)

      AERGn(i,j) = AERG(i,j)
      
      enddo loopmuljn
      enddo loopmulin


      
      loopmuli: do i = 1, 3
      loopmulj: do j = 1, 3

      AURG(i,j) = AURG(i,j) * yuRG(i,j)
      
      ADRG(i,j) = ADRG(i,j) * ydRG(i,j)

      AERG(i,j) = AERG(i,j) * yeRG(i,j)
      
      enddo loopmulj
      enddo loopmuli
      
      AOK = 0




C     Mass Matrices
C     ---------------------------------------------------------------
C     Up-Squark Mass Matrix 
C     ---------------------------------------------------------------

      mSU(1,1) = mSQRG(1,1) + mUQ*mUQ + MZ*MZ*cos2beta*(0.5d0 - 
     .	         (2.d0/3.d0)*sinsqtw)
      mSU(1,2) = mSQRG(1,2) 
      mSU(1,3) = mSQRG(1,3)
      mSU(1,4) = - AURG(1,1)*vev2/dsqrt(2.d0)-
     $     mUQ*sgnmu*mur*(1.d0/tanbeta) 
      mSU(1,5) = - AURG(1,2)*vev2/dsqrt(2.d0) 
      mSU(1,6) = - AURG(1,3)*vev2/dsqrt(2.d0)
      mSU(2,1) = mSQRG(2,1)
      mSU(2,2) = mSQRG(2,2) + mC*mC + MZ*MZ*cos2beta*(0.5d0 -
     .     (2.d0/3.d0)*sinsqtw)
      mSU(2,3) = mSQRG(2,3)
      mSU(2,4) = - AURG(2,1)*vev2/dsqrt(2.d0) 
      mSU(2,5) = - AURG(2,2)*vev2/dsqrt(2.d0)-
     $     mC*sgnmu*mur*(1.d0/tanbeta)
      mSU(2,6) = - AURG(2,3)*vev2/dsqrt(2.d0) 
      mSU(3,1) = mSQRG(3,1)
      mSU(3,2) = mSQRG(3,2)
      mSU(3,3) = mSQRG(3,3) + mT*mT + MZ*MZ*cos2beta*(0.5d0 
     $     - (2.d0/3.d0)*sinsqtw)
      mSU(3,4) = - AURG(3,1)*vev2/dsqrt(2.d0) 
      mSU(3,5) = - AURG(3,2)*vev2/dsqrt(2.d0) 
      mSU(3,6) = - AURG(3,3)*vev2/dsqrt(2.d0) - 
     $     mT*sgnmu*mur*(1.d0/tanbeta)
      mSU(4,1) = - AURG(1,1)*vev2/dsqrt(2.d0) - 
     $     mUQ*sgnmu*mur*(1.d0/tanbeta)
      mSU(4,2) = - AURG(2,1)*vev2/dsqrt(2.d0)
      mSU(4,3) = - AURG(3,1)*vev2/dsqrt(2.d0)
      mSU(4,4) = mSURG(1,1)+mUQ*mUQ+(2.d0/3.d0)*MZ*MZ*cos2beta*sinsqtw
      mSU(4,5) = mSURG(1,2)
      mSU(4,6) = mSURG(1,3)
      mSU(5,1) = - AURG(1,2)*vev2/dsqrt(2.d0)
      mSU(5,2) = - AURG(2,2)*vev2/dsqrt(2.d0) - 
     $     mC*sgnmu*mur*(1.d0/tanbeta)
      mSU(5,3) = - AURG(3,2)*vev2/dsqrt(2.d0)
      mSU(5,4) = mSURG(2,1)
      mSU(5,5) = mSURG(2,2)+mC*mC+(2.d0/3.d0)*MZ*MZ*cos2beta*sinsqtw
      mSU(5,6) = mSURG(2,3)
      mSU(6,1) = - AURG(1,3)*vev2/dsqrt(2.d0)
      mSU(6,2) = - AURG(2,3)*vev2/dsqrt(2.d0)
      mSU(6,3) = - AURG(3,3)*vev2/dsqrt(2.d0) - 
     $     mT*sgnmu*mur*(1.d0/tanbeta)
      mSU(6,4) = mSURG(3,1)
      mSU(6,5) = mSURG(3,2)
      mSU(6,6) = mSURG(3,3) + mT*mT + (2.d0/3.d0)*MZ*MZ*cos2beta*sinsqtw

      
C     Make a copy of the Mass Matrix. 
      
      do a = 1,6
         do m = 1,6 
            
           mSU1(a,m) = mSU(a,m)
           mSU2(a,m) = mSU(a,m)

           
        enddo
      enddo 


!-------------------------------------------------------------------------------      
C     find  Eigenvalues using Lapacks eigenvalue routine. 
            
      
C     find singular values and diaongalising matrices. 
      
      if(scale.gt.92.d0)then

      info = 10
      
      call dgesvd('A','A',6,6,mSU1,6,SUegvd,USUvd,6,USUT,6,
     $     work,lwork,info)
      
      if(info.eq.0) then
         AOK = AOK + 1
      endif

C     According to definition take the transpose for the diagonalising matrix.
      
      do a = 1, 6
 	 do m = 1, 6 
            
            USUvd(a,m) = USUT(a,m)
            
 	 enddo  
      enddo 

      
      endif


!---------------------------------
C    squark sector, 3rd gen

         stops(1,1) = mSU2(3,3)
         stops(1,2) = mSU2(3,6)
         stops(2,1) = mSU2(6,3)
         stops(2,2) = mSU2(6,6)



      call DGEEVX( 'N', 'V', 'V', 'N', 2, stops, 2, stopegg, Stopsi,
     $                   stopL, 2, stopR, 2, ILO, IHI, SCALE1, ABNRM,
     $                   RCONDE, RCONDV, WORK, LWORK, IWORK, INFO )

!---2nd gen

         schs(1,1) = mSU2(2,2)
         schs(1,2) = mSU2(2,5)
         schs(2,1) = mSU2(5,2)
         schs(2,2) = mSU2(5,5)


      call DGEEVX( 'N', 'V', 'V', 'N', 2, schs, 2, schegg, Schsi,
     $                   schL, 2, schR, 2, ILO, IHI, SCALE1, ABNRM,
     $                   RCONDE, RCONDV, WORK, LWORK, IWORK, INFO )

!---1st gen

         sups(1,1) = mSU2(1,1)
         sups(1,2) = mSU2(1,4)
         sups(2,1) = mSU2(4,1)
         sups(2,2) = mSU2(4,4)
         

      call DGEEVX( 'N', 'V', 'V', 'N', 2, sups, 2, supegg, Supsi,
     $                   supL, 2, supR, 2, ILO, IHI, SCALE1, ABNRM,
     $                   RCONDE, RCONDV, WORK, LWORK, IWORK, INFO )

!--------------------------------------------------------------------------

       if(scale.gt.92.d0)then

          thetat = dacos(dabs(stopL(1,1)))
          thetac = dacos(dabs(schL(1,1)))
          thetau = dacos(dabs(supL(1,1)))
       else
          thetatz = dacos(dabs(stopL(1,1)))
          thetacz = dacos(dabs(schL(1,1)))
          thetauz = dacos(dabs(supL(1,1)))
       endif



      if(dabs(stopr(1,1)).gt.dabs(stopr(2,1)))then
         SUegg(1) = stopegg(1)
         SUegg(2) = stopegg(2)
      else
         SUegg(1) = stopegg(2)
         SUegg(2) = stopegg(1)
      endif
      

      if(dabs(schr(1,1)).gt.dabs(schr(2,1)))then
         SUegg(3) = schegg(1)
         SUegg(4) = schegg(2)
      else
         SUegg(3) = schegg(2)
         SUegg(4) = schegg(1)
      endif



      if(dabs(supr(1,1)).gt.dabs(supr(2,1)))then
         SUegg(5) = supegg(1)
         SUegg(6) = supegg(2)
      else
         SUegg(5) = supegg(2)
         SUegg(6) = supegg(1)
      endif


      traceT = MSU2(6,6) + MSU2(3,3)
      DETT = MSU2(6,6)* MSU2(3,3) - MSU2(3,6)**2.D0

      eigt1 = (traceT + dsqrt(traceT**2.d0 - 4.d0*DETT ))/2.d0
  
      eigt2 = (traceT - dsqrt(traceT**2.d0 - 4.d0*DETT ))/2.d0

      SUegg(1) = eigt1
      SUegg(2) = eigt2


      
C     ---------------------t-------------------------------------------
C     Down-Squark Mass Matrix 
C     ---------------------------------------------------------------

      mSD(1,1) = mSQRG(1,1) + mD*mD + MZ*MZ*cos2beta*(- 0.5d0 + 
     .	          (1.d0/3.d0)*sinsqtw) 
      mSD(1,2) = mSQRG(1,2) 
      mSD(1,3) = mSQRG(1,3)
      mSD(1,4) = ADRG(1,1)*vev1/dsqrt(2.d0) - mD*sgnmu*mur*tanbeta 
      mSD(1,5) = ADRG(1,2)*vev1/dsqrt(2.d0) 
      mSD(1,6) = ADRG(1,3)*vev1/dsqrt(2.d0)
      mSD(2,1) = mSQRG(2,1)
      mSD(2,2) = mSQRG(2,2) + mS*mS + MZ*MZ*cos2beta*(- 0.5d0 + 
     . 	          (1.d0/3.d0)*sinsqtw)
      mSD(2,3) = mSQRG(2,3)
      mSD(2,4) = ADRG(2,1)*vev1/dsqrt(2.d0) 
      mSD(2,5) = ADRG(2,2)*vev1/dsqrt(2.d0) - mS*sgnmu*mur*tanbeta
      mSD(2,6) = ADRG(2,3)*vev1/dsqrt(2.d0) 
      mSD(3,1) = mSQRG(3,1)
      mSD(3,2) = mSQRG(3,2)
      mSD(3,3) = mSQRG(3,3) + mB*mB + MZ*MZ*cos2beta*(- 0.5d0 + 
     .            (1.d0/3.d0)*sinsqtw)
      mSD(3,4) = ADRG(3,1)*vev1/dsqrt(2.d0) 
      mSD(3,5) = ADRG(3,2)*vev1/dsqrt(2.d0) 
      mSD(3,6) = ADRG(3,3)*vev1/dsqrt(2.d0) - mB*sgnmu*mur*tanbeta
      mSD(4,1) = ADRG(1,1)*vev1/dsqrt(2.d0) - mD*sgnmu*mur*tanbeta
      mSD(4,2) = ADRG(2,1)*vev1/dsqrt(2.d0)
      mSD(4,3) = ADRG(3,1)*vev1/dsqrt(2.d0)
      mSD(4,4) = mSDRG(1,1) + mD*mD - (1.d0/3.d0)*MZ*MZ*
     .                  cos2beta*sinsqtw
      mSD(4,5) = mSDRG(1,2)
      mSD(4,6) = mSDRG(1,3)
      mSD(5,1) = ADRG(1,2)*vev1/dsqrt(2.d0)
      mSD(5,2) = ADRG(2,2)*vev1/dsqrt(2.d0) - mS*sgnmu*mur*tanbeta
      mSD(5,3) = ADRG(3,2)*vev1/dsqrt(2.d0)
      mSD(5,4) = mSDRG(2,1)
      mSD(5,5) = mSDRG(2,2) + mS*mS - (1.d0/3.d0)*MZ*MZ*
     .            cos2beta*sinsqtw
      mSD(5,6) = mSDRG(2,3)
      mSD(6,1) = ADRG(1,3)*vev1/dsqrt(2.d0)
      mSD(6,2) = ADRG(2,3)*vev1/dsqrt(2.d0)
      mSD(6,3) = ADRG(3,3)*vev1/dsqrt(2.d0) - mB*sgnmu*mur*tanbeta
      mSD(6,4) = mSDRG(3,1)
      mSD(6,5) = mSDRG(3,2)
      mSD(6,6) = mSDRG(3,3) + mB*mB - (1.d0/3.d0)*MZ*MZ*
     .            cos2beta*sinsqtw


C     --------------------------------------------------------
C     Make a copy of the Mass Matrix. 

      do a = 1, 6
         do m = 1, 6 
            

            mSD1(a,m) = mSD(a,m)
            mSD2(a,m) = mSD(a,m)
            mSD3(a,m) = mSD(a,m)

         enddo
      enddo 

C     find sdown sector Eigenvalues using Lapacks eigenvalue routine. 
      

C     find singular values and diaongalising matrices. 
      
      info = 10
      
      if(scale.gt.92.d0)then
       call dgesvd('A','A',6,6,mSD3,6,SDegvd,USDvd,6,USDT,6,
     .	work,lwork,info)


C     According to definition take the transpose for the diagonalising matrix.
      
      do a = 1, 6
 	 do m = 1, 6 
            
            USDvd(a,m) = USDT(a,m)
            
 	 enddo  
      enddo 


         endif

!--------------------
        
        

         sbtms(1,1) = mSD2(3,3)
         sbtms(1,2) = mSD2(3,6)
         sbtms(2,1) = mSD2(6,3)
         sbtms(2,2) = mSD2(6,6)



      call DGEEVX( 'N', 'V', 'V', 'N', 2, sbtms, 2, sbtmegg, Sbtmsi,
     $                   sbtmL, 2, sbtmR, 2, ILO, IHI, SCALE1, ABNRM,
     $                   RCONDE, RCONDV, WORK, LWORK, IWORK, INFO )

!---2nd gen

         sstrs(1,1) = mSD2(2,2)
         sstrs(1,2) = mSD2(2,5)
         sstrs(2,1) = mSD2(5,2)
         sstrs(2,2) = mSD2(5,5)


      call DGEEVX( 'N', 'V', 'V', 'N', 2, sstrs, 2, sstregg, Sstrsi,
     $                   sstrL, 2, sstrR, 2, ILO, IHI, SCALE1, ABNRM,
     $                   RCONDE, RCONDV, WORK, LWORK, IWORK, INFO )

!---1st gen

         sdowns(1,1) = mSD2(1,1)
         sdowns(1,2) = mSD2(1,4)
         sdowns(2,1) = mSD2(4,1)
         sdowns(2,2) = mSD2(4,4)
         

      call DGEEVX( 'N', 'V', 'V', 'N', 2, sdowns, 2, sdownegg, Sdownsi,
     $                   sdownL, 2, sdownR, 2, ILO, IHI, SCALE1, ABNRM,
     $                   RCONDE, RCONDV, WORK, LWORK, IWORK, INFO )

!---------------------------------------------------------------------

       if(scale.gt.92.d0)then

          thetab = dacos((sbtmL(1,1)))
          thetas = dacos((sstrL(1,1)))
          thetad = dacos((sdownL(1,1)))
       else
          thetabz = dacos((sbtmL(1,1)))
          thetasz = dacos((sstrL(1,1)))
          thetadz = dacos((sdownL(1,1)))
       endif

       if(scale.gt.92.d0)then

          thetab = dacos(dabs(sbtmL(1,1)))
          thetas = dacos(dabs(sstrL(1,1)))
          thetad = dacos(dabs(sdownL(1,1)))
       else
          thetabz = dacos(dabs(sbtmL(1,1)))
          thetasz = dacos(dabs(sstrL(1,1)))
          thetadz = dacos(dabs(sdownL(1,1)))
       endif


      if(dabs(sbtmr(1,1)).gt.dabs(sbtmr(2,1)))then
         SDegg(1) = sbtmegg(1)
         SDegg(2) = sbtmegg(2)
      else
         SDegg(1) = sbtmegg(2)
         SDegg(2) = sbtmegg(1)
      endif
      

      if(dabs(sstrr(1,1)).gt.dabs(sstrr(2,1)))then
         SDegg(3) = sstregg(1)
         SDegg(4) = sstregg(2)
      else
         SDegg(3) = sstregg(2)
         SDegg(4) = sstregg(1)
      endif


      if(dabs(sdownr(1,1)).gt.dabs(sdownr(2,1)))then
         SDegg(5) = sdownegg(1)
         SDegg(6) = sdownegg(2)
      else
         SDegg(5) = sdownegg(2)
         SDegg(6) = sdownegg(1)
      endif



      traceB = MSD2(6,6) + MSD2(3,3)
      DETB = MSD2(6,6)* MSD2(3,3) - MSD2(3,6)**2.D0

      eigb1 = (traceb + dsqrt(traceb**2.d0 - 4.d0*DETB ))/2.d0
  
      eigb2 = (traceb - dsqrt(traceb**2.d0 - 4.d0*DETB ))/2.d0

      SDegg(1) = eigb2
      SDegg(2) = eigb1



C     --------------------------------------------------------
C     Slepton Mass Matrix
C     --------------------------------------------------------


      mSL(1,1) = mSLRG(1,1)+mE*mE+MZ*MZ*cos2beta*(- 0.5d0 + sinsqtw) 
      mSL(1,2) = mSLRG(1,2) 
      mSL(1,3) = mSLRG(1,3)
      mSL(1,4) = AERG(1,1)*vev1/dsqrt(2.d0) - mE*sgnmu*mur*tanbeta 
      mSL(1,5) = AERG(1,2)*vev1/dsqrt(2.d0) 
      mSL(1,6) = AERG(1,3)*vev1/dsqrt(2.d0)
      mSL(2,1) = mSLRG(2,1)
      mSL(2,2) = mSLRG(2,2)+mMU*mMU+MZ*MZ*cos2beta*(-0.5d0 + sinsqtw)
      mSL(2,3) = mSLRG(2,3)
      mSL(2,4) = AERG(2,1)*vev1/dsqrt(2.d0) 
      mSL(2,5) = AERG(2,2)*vev1/dsqrt(2.d0) - mMU*sgnmu*mur*tanbeta
      mSL(2,6) = AERG(2,3)*vev1/dsqrt(2.d0) 
      mSL(3,1) = mSLRG(3,1)
      mSL(3,2) = mSLRG(3,2)
      mSL(3,3) = mSLRG(3,3)+mTAU*mTAU+MZ*MZ*cos2beta*(-0.5d0+sinsqtw)
      mSL(3,4) = AERG(3,1)*vev1/dsqrt(2.d0) 
      mSL(3,5) = AERG(3,2)*vev1/dsqrt(2.d0) 
      mSL(3,6) = AERG(3,3)*vev1/dsqrt(2.d0) - mTAU*sgnmu*mur*tanbeta
      mSL(4,1) = AERG(1,1)*vev1/dsqrt(2.d0) - mE*sgnmu*mur*tanbeta
      mSL(4,2) = AERG(2,1)*vev1/dsqrt(2.d0)
      mSL(4,3) = AERG(3,1)*vev1/dsqrt(2.d0)
      mSL(4,4) = mSERG(1,1) + mE*mE - MZ*MZ*cos2beta*sinsqtw
      mSL(4,5) = mSERG(1,2)
      mSL(4,6) = mSERG(1,3)
      mSL(5,1) = AERG(1,2)*vev1/dsqrt(2.d0)
      mSL(5,2) = AERG(2,2)*vev1/dsqrt(2.d0) - mMU*sgnmu*mur*tanbeta
      mSL(5,3) = AERG(3,2)*vev1/dsqrt(2.d0)
      mSL(5,4) = mSERG(2,1)
      mSL(5,5) = mSERG(2,2) + mMU*mMU - MZ*MZ*cos2beta*sinsqtw
      mSL(5,6) = mSERG(2,3) 
      mSL(6,1) = AERG(1,3)*vev1/dsqrt(2.d0)
      mSL(6,2) = AERG(2,3)*vev1/dsqrt(2.d0)
      mSL(6,3) = AERG(3,3)*vev1/dsqrt(2.d0) - mTAU*sgnmu*mur*tanbeta
      mSL(6,4) = mSERG(3,1)
      mSL(6,5) = mSERG(3,2) 
      mSL(6,6) = mSERG(3,3) + mTAU*mTAU - MZ*MZ*cos2beta*sinsqtw


C	-------------------------------------------------------------

C     Make a copy of the Mass Matrix. 

      do a = 1,6
         do m = 1,6 
            
            mSL1(a,m) = mSL(a,m)
            mSL2(a,m) = mSL(a,m)
            mSL3(a,m) = mSL(a,m)

         enddo
      enddo 



C     find singular values and diaongalising matrices. 
      
      info = 10

       by66:  if(scale.gt.92.d0)then

      call dgesvd('A','A',6,6,mSL1,6,SLegvd,USLvd,6,USLT,6,
     $     work,lwork,info)

!------------------------------------
C     According to definition take the transpose for the diagonalising matrix.

      do a = 1, 6
 	 do m = 1,6 
            
            USLvd(a,m) = USLT(a,m)
            
 	 enddo
      enddo 


      call DGEEVX( 'N', 'V', 'V', 'N', 6, mSL3, 6, slegs, slegi,
     $                   USLs, 6, USLTs, 6, ILO, IHI, SCALE1, ABNRM,
     $                   RCONDE, RCONDV, WORK, LWORK, IWORK, INFO )
      

      if(info.eq.0) then
         AOK = AOK + 1
      endif



!---------------------------------
C     Lepton sector, full 6x6


c$$$        call sorteign(USLTs,SLegs,slpmix,slpmixT,slpegn)
c$$$
c$$$        do i = 1,6
c$$$           do j = 1,6
c$$$              USLTsrt(i,j) = slpmixT(i,j)
c$$$           enddo
c$$$        enddo

      endif by66



!-----2x2 slepton

         staus(1,1) = mSL2(3,3)
         staus(1,2) = mSL2(3,6)
         staus(2,1) = mSL2(6,3)
         staus(2,2) = mSL2(6,6)


C     find SLepton Eigenvalues using Lapacks eigenvalue routine. 

      call DGEEVX( 'N', 'V', 'V', 'N', 2, staus, 2, staugg, Stausi,
     $                   stauL, 2, stauR, 2, ILO, IHI, SCALE1, ABNRM,
     $                   RCONDE, RCONDV, WORK, LWORK, IWORK, INFO )


         smus(1,1) = mSL2(2,2)
         smus(1,2) = mSL2(2,5)
         smus(2,1) = mSL2(5,2)
         smus(2,2) = mSL2(5,5)


C     find SLepton Eigenvalues using Lapacks eigenvalue routine. 


      call DGEEVX( 'N', 'V', 'V', 'N', 2, smus, 2, smugg, Smusi,
     $                   smuL, 2, smuR, 2, ILO, IHI, SCALE1, ABNRM,
     $                   RCONDE, RCONDV, WORK, LWORK, IWORK, INFO )


         sels(1,1) = mSL2(1,1)
         sels(1,2) = mSL2(1,4)
         sels(2,1) = mSL2(4,1)
         sels(2,2) = mSL2(4,4)


C     find SLepton Eigenvalues using Lapacks eigenvalue routine. 


      call DGEEVX( 'N', 'V', 'V', 'N', 2, sels, 2, selgg, Selsi,
     $                   selL, 2, selR, 2, ILO, IHI, SCALE1, ABNRM,
     $                   RCONDE, RCONDV, WORK, LWORK, IWORK, INFO )

       if(scale.gt.92.d0)then

          thetatau = dacos(dabs(stauL(1,1)))
          thetamu = dacos(dabs(smuL(1,1)))
          thetae = dacos(dabs(selL(1,1)))
       else
          thetatauz = dacos(dabs(stauL(1,1)))
          thetamuz = dacos(dabs(smuL(1,1)))
          thetaez = dacos(dabs(selL(1,1)))
       endif


      if(dabs(staur(1,1)).gt.dabs(staur(2,1)))then
         SLegg(1) = staugg(1)
         SLegg(2) = staugg(2)
      else
         SLegg(1) = staugg(2)
         SLegg(2) = staugg(1)
      endif
      

      if(dabs(smur(1,1)).gt.dabs(smur(2,1)))then
         SLegg(3) = smugg(1)
         SLegg(4) = smugg(2)
      else
         SLegg(3) = smugg(2)
         SLegg(4) = smugg(1)
      endif


      if(dabs(selr(1,1)).gt.dabs(selr(2,1)))then
         SLegg(5) = selgg(1)
         SLegg(6) = selgg(2)
      else
         SLegg(5) = selgg(2)
         SLegg(6) = selgg(1)
      endif


      traceL = MSL2(6,6) + MSL2(3,3)
      DETL = MSL2(6,6)* MSL2(3,3) - MSL2(3,6)**2.D0

      eigl1 = (traceL + dsqrt(traceL**2.d0 - 4.d0*DETL ))/2.d0
  
      eigl2 = (traceL - dsqrt(traceL**2.d0 - 4.d0*DETL ))/2.d0

      SLegg(1) = eigl1
      SLegg(2) = eigl2

!---------------------------------
C     ------------------------------------------------------------------	
C     Sneutrino Mass Matrix
C     -------------------------------------------------------------------

      mSN(1,1) = mSLRG(1,1) + 0.5d0*MZ*MZ*cos2beta
      mSN(1,2) = mSLRG(1,2)
      mSN(1,3) = mSLRG(1,3)
      mSN(2,1) = mSLRG(2,1)
      mSN(2,2) = mSLRG(2,2) + 0.5d0*MZ*MZ*cos2beta
      mSN(2,3) = mSLRG(2,3)
      mSN(3,1) = mSLRG(3,1)
      mSN(3,2) = mSLRG(3,2)
      mSN(3,3) = mSLRG(3,3) + 0.5d0*MZ*MZ*cos2beta

C     Make a copy of the Mass Matrix. 

      do a = 1,3
         do m = 1,3 
            
            mSN1(a,m) = mSN(a,m)

         enddo
      enddo 


      info = 10

C     find sneutrino eigenvalues 

      call dsyev('V','U',3,mSN,3,SNegg,work1,lwork1,info)

      if(info.eq.0)then
         AOK = AOK + 1
      endif 

C     find the singular values and the diagonalising matrix.

      info = 10
      
      call dgesvd('A','A',3,3,mSN1,3,SNeg,USN,3,USNT,3,
     .     work1,lwork1,info)
      
      if(info.eq.0) then
         AOK = AOK + 1
      endif

C     Take the transpose as per the defintion.

      do a = 1,3
         do m = 1,3 
            
            USN(a,m) = USNT(a,m)

         enddo
      enddo

c$$$      print*,"Sneg(1),Sneg(2),Sneg(3)",Sneg(1),
c$$$     $     Sneg(2),Sneg(3)

C     --------------------------------------------------------
C     Neutralino Mass Matrix 
C     ---------------------------------------------------------

      MNeut(1,1) = M1tz
      MNeut(1,2) = 0.d0
      MNeut(1,3) = - (MZ*stw*cbeta)
      MNeut(1,4) = MZ*stw*sbeta
      MNeut(2,1) = 0.d0
      MNeut(2,2) = M2tz
      MNeut(2,3) = MZ*ctw*cbeta
      MNeut(2,4) = -(MZ*ctw*sbeta)
      MNeut(3,1) = -(MZ*stw*cbeta)
      MNeut(3,2) =  MZ*ctw*cbeta
      MNeut(3,3) = 0.d0
      MNeut(3,4) = - sgnmu*mur
      MNeut(4,1) = MZ*stw*sbeta
      MNeut(4,2) = - (MZ*ctw*sbeta)
      MNeut(4,3) = - sgnmu*mur 
      MNeut(4,4) = 0.d0 


!-------------------------------------------------------------------

C     Make a copy of the Mass Matrix. 

      do a = 1, 4
         do m = 1, 4 
            
            MNeut1(a,m) = MNeut(a,m)
            MNeut2(a,m) = MNeut(a,m)

         enddo
      enddo 

C     find singular values and diagonalising matrix.

      call DGEEV( 'V', 'V', 4, MNeut2, 4, Neg, Neuevi, ONL, 4, 
     $     ONR, 4, WORK, LWORK, INFO )
      
      if(info.eq.0) then
         AOK = AOK + 1
      endif
      
      do a = 1,4
         do m = 1,4 
            
            ON(a,m) = ONR(m,a)
            
         enddo
      enddo



C     ----------------------------------------------------------------
C     Chargino Mass Matrix
C     ----------------------------------------------------------------

      MChar(1,1) = M2tz  
      MChar(1,2) = sqrt(2.d0)*MW*sbeta
      MChar(2,1) = sqrt(2.d0)*MW*cbeta
      MChar(2,2) = sgnmu*mur
      
      
       call dgesvd('A','A',2,2,MChar,2,Ceg,OCL,2,OCRT,2,
     .	work1,lwork1,info)

      

      do a = 1, 2
         do m = 1,2 
            
            OCR(m,a) = OCRT(a,m)


         enddo
      enddo 
      


C-----------------------------------------------------------------
C     Tree level Higgs mass spectrum 
C-------------------------------------------------------------------

       if(bmur.lt.0.d0) then
          mA0sq =   (dsqrt(dabs(SUegg(1)))*
     $         dsqrt(dabs(SUegg(2))) )/(16.d0 * pi *pi) 

          flag_bmu = ' -ve Bmu'
       else
          mA0sq = bmur/(sbeta*cbeta)
          flag_bmu = ' AOK'
       endif

!----------
        mhpmsq = mA0sq + MW*MW


        Mhiggstree(1,1) = mA0sq*dsin(beta)**2.d0 + 
     $       (MZ*MZ)*dcos(beta)**2.d0

        Mhiggstree(1,2) = -1.d0*(mA0sq + 
     $       (MZ*MZ)) * dsin(beta)*dcos(beta)
        
        Mhiggstree(2,1) = Mhiggstree(1,2)

        Mhiggstree(2,2) = mA0sq*dcos(beta)**2.d0 + 
     $       (MZ*MZ)*dsin(beta)**2.d0

        Mhiggsn(1,1) = Mhiggstree(1,1) 

        Mhiggsn(1,2) = Mhiggstree(1,2) 

        Mhiggsn(2,1) = Mhiggstree(2,1) 

        Mhiggsn(2,2) = Mhiggstree(2,2) 

        alphatree    = 0.d0 + 
     $       datan((2.d0*Mhiggsn(1,2))/(Mhiggsn(1,1)-Mhiggsn(2,2)))
     $       *0.5d0


      call DGEEV('V', 'V', 2, Mhiggsn, 2, heign, lhim, hl, 2, 
     $     hr, 2, WORK, LWORK, INFO)

!----------------


        mh0sq = heign(2)       
        mhu0sq = heign(1)      

      loopmulin1: do i = 1, 3
      loopmuljn1: do j = 1, 3

      AURG(i,j) = AURGn(i,j)
      
      ADRG(i,j) = ADRGn(i,j)

      AERG(i,j) = AERGn(i,j)
      
      enddo loopmuljn1
      enddo loopmulin1

       return 

       end subroutine softspectrum 
!=================================================================================
!
!
!    Computes tree level \mu and b\mu. 
!
!-----------------------------------------------------------------------

      SUBROUTINE mutreelevel(tanbetaQ,mh1mz,mh2mz,mursq,mur,bmur,
     $     flagdet,flagDflat)

      double precision  cbeta, sbeta,beta
      double precision sin2beta, tan2beta
      double precision  cos2beta, tanbetaQ
      DOUBLE PRECISION mh1mz,mh2mz,mur,mursq,bmur,mh1sq,mh2sq
      integer flagdet, flagDflat
      double precision detisneg,nodflat
      DOUBLE PRECISION m0, m12, m10, m20, sgnmu, tanbeta, a0
      common/mssminputs/ m0, m12, m10, m20, sgnmu, tanbeta, a0

!-----------------------------------------------------------------

      include 'stdinputs.h'
c-----------------------------------------------------------------
C     MU and bmu Part - Tree Level
C ----------------------------------------------------------------

      beta  = datan(tanbetaQ)
      sbeta = dsin(beta)
      cbeta = dcos(beta)
      
      cos2beta = cbeta*cbeta - sbeta*sbeta
      sin2beta = 2.d0*cbeta*sbeta
      tan2beta = sin2beta/cos2beta
      
      mh1sq = mh1mz
      mh2sq = mh2mz

      flagdet = 0
      flagDflat = 0

C--------------------------------------------------------------------

	mursq = (((mh1sq*sbeta*sbeta - mh2sq*cbeta*cbeta)/cos2beta)
     $       - 0.5d0*MZ*MZ)

        bmur = (0.5d0*sin2beta*(mh2sq + mh1sq + 2.d0*mursq))
            
C--------------------------------------------------------------------
!     no D flat directions or detisneg (TL)?
!---------------------------------------------------------

	noDflat = 2.d0*(mursq - bmur) + mh2sq + mh1sq 

	detisneg = (mursq+mh1sq)*(mursq+mh2sq) - bmur*bmur

	sud1if: if(detisneg.lt.0.d0)then
           flagdet = 1                 !<---------det is negative satisfied
 !       write(*,*)'detisneg '
	endif sud1if

	sud2if: if(noDflat.gt.0.d0)then
           flagDflat = 1               !<-----------Dflat satisfied
  !      write(*,*)'dflat satisfied'

	endif sud2if


C--------------------------------------------
C     choosing sign of \mu
C-------------------------------------------


        muge:  if(mursq.ge.0)then
                                     
              mur = dsqrt(mursq)

        else 

           mur = dsqrt(dabs(mursq))
           
        endif muge
      

        RETURN

        END SUBROUTINE mutreelevel
C=========================================================================================
C=========================================================================================

      
      Subroutine sorteign(A,eign,B,BT,seign)

      implicit none

      integer i,sl1pos,sl2pos,melpos,merpos,mmulpos,mmurpos
      integer j,slep1,slep2,slep3,slep4,slep5,slep6,m3rpos
      integer slep1c,slep2c,slep3c,slep4c,slep5c,slep6c

      double precision A(6,6),B(6,6),BT(6,6),seign(6),eign(6)
      DOUBLE PRECISION sl1(6),sl2(6),sl3(6),sl4(6),sl5(6),sl6(6)
      DOUBLE PRECISION sl1c(6),sl2c(6),sl3c(6),sl4c(6),sl5c(6),sl6c(6)
      
      double precision sl1val,slepa(6)
      character*1 flip
      double precision slcnew(6,4),slc(6,6),slcnewconf(6,2)
      double precision slegsnew(4),slegsnewconf(2)
      integer counter1,counter2,count(2)
      integer countempty,empty(2)

!      data B/ 36 * 0.d0/, Bt/ 36 * 0.d0/, seign/ 6 * 0.d0/
      
      
!------------------



      sl1val = MINVAL(eign, dim = 1)
      sl1pos = MINLOC(eign, DIM = 1)


      do i = 1,6
         sl1(i) = dabs(A(i,1))
         sl2(i) = dabs(A(i,2))
         sl3(i) = dabs(A(i,3))
         sl4(i) = dabs(A(i,4))
         sl5(i) = dabs(A(i,5))
         sl6(i) = dabs(A(i,6))    


         slc(i,1) = A(i,1)
         slc(i,2) = A(i,2)
         slc(i,3) = A(i,3)
         slc(i,4) = A(i,4)
         slc(i,5) = A(i,5)
         slc(i,6) = A(i,6)           
         
      enddo
      
      slep1 = MAXLOC(sl1, DIM = 1)
      slep2 = MAXLOC(sl2, DIM = 1)
      slep3 = MAXLOC(sl3, DIM = 1)
      slep4 = MAXLOC(sl4, DIM = 1)
      slep5 = MAXLOC(sl5, DIM = 1)
      slep6 = MAXLOC(sl6, DIM = 1)

      slepa(1) = slep1
      slepa(2) = slep2
      slepa(3) = slep3
      slepa(4) = slep4
      slepa(5) = slep5
      slepa(6) = slep6

c$$$      print*,'slep1 = ', slep1
c$$$      print*,'slep2 = ', slep2
c$$$      print*,'slep3 = ', slep3
c$$$      print*,'slep4 = ', slep4
c$$$      print*,'slep5 = ', slep5
c$$$      print*,'slep6 = ', slep6

      count(1) = 0
      count(2) = 0
      flip = 'F'

      do i = 1,6
         do j= 1,6
            B(i,j) = 0.d0
            BT(i,j) = 0.d0
            seign(i) = 0.d0
            if(i.ne.j)then
               if(slepa(i).eq.slepa(j))then
                  count(1) = i
                  count(2) = j
                  flip='T'
               endif
            endif
         enddo
      enddo




!===============================================
!     When Flip is "FALSE"
!----------------------pos 1

      if(flip.eq.'F')then
         

         if(slep1.eq.1)then

            seign(5) = eign(1)

            do i = 1,6
               BT(5,i) = A(i,1) 
            enddo

         elseif(slep1.eq.2)then
            
            seign(3) = eign(1)

            do i = 1,6
               BT(3,i) = A(i,1) 
            enddo

         elseif(slep1.eq.3)then
            
            seign(1) = eign(1)
            do i = 1,6
               BT(1,i) = A(i,1) 
            enddo

         elseif(slep1.eq.4)then
            
            seign(6) = eign(1)
            do i = 1,6
               BT(6,i) = A(i,1) 
            enddo

         elseif(slep1.eq.5)then
            
            seign(4) = eign(1)
            do i = 1,6
               BT(4,i) = A(i,1) 
            enddo

         elseif(slep1.eq.6)then
            
            seign(2) = eign(1)
            do i = 1,6
               BT(2,i) = A(i,1) 
            enddo

         endif

!------pos2

         if(slep2.eq.1)then

            seign(5) = eign(2)
            do i = 1,6
               BT(5,i) = A(i,2) 
            enddo

         elseif(slep2.eq.2)then
            
            seign(3) = eign(2)
            do i = 1,6
               BT(3,i) = A(i,2) 
            enddo

         elseif(slep2.eq.3)then
            
            seign(1) = eign(2)
            do i = 1,6
               BT(1,i) = A(i,2) 
            enddo

         elseif(slep2.eq.4)then
            
            seign(6) = eign(2)
            do i = 1,6
               BT(6,i) = A(i,2) 
            enddo

         elseif(slep2.eq.5)then
            
            seign(4) = eign(2)
            do i = 1,6
               BT(4,i) = A(i,2) 
            enddo

         elseif(slep2.eq.6)then
            
            seign(2) = eign(2)
            do i = 1,6
               BT(2,i) = A(i,2) 
            enddo

         endif


!------pos3

         if(slep3.eq.1)then

            seign(5) = eign(3)
            do i = 1,6
               BT(5,i) = A(i,3) 
            enddo

         elseif(slep3.eq.2)then
            
            seign(3) = eign(3)
            do i = 1,6
               BT(3,i) = A(i,3) 
            enddo

         elseif(slep3.eq.3)then
            
            seign(1) = eign(3)
            do i = 1,6
               BT(1,i) = A(i,3) 
            enddo

         elseif(slep3.eq.4)then
            
            seign(6) = eign(3)
            do i = 1,6
               BT(6,i) = A(i,3) 
            enddo

         elseif(slep3.eq.5)then
            
            seign(4) = eign(3)
            do i = 1,6
               BT(4,i) = A(i,3) 
            enddo

         elseif(slep3.eq.6)then
            
            seign(2) = eign(3)
            do i = 1,6
               BT(2,i) = A(i,3) 
            enddo

         endif

!------pos4

         if(slep4.eq.1)then

            seign(5) = eign(4)
            do i = 1,6
               BT(5,i) = A(i,4) 
            enddo

         elseif(slep4.eq.2)then
            
            seign(3) = eign(4)
            do i = 1,6
               BT(3,i) = A(i,4) 
            enddo

         elseif(slep4.eq.3)then
            
            seign(1) = eign(4)
            do i = 1,6
               BT(1,i) = A(i,4) 
            enddo

         elseif(slep4.eq.4)then
            
            seign(6) = eign(4)
            do i = 1,6
               BT(6,i) = A(i,4) 
            enddo

         elseif(slep4.eq.5)then
            
            seign(4) = eign(4)
            do i = 1,6
               BT(4,i) = A(i,4) 
            enddo

         elseif(slep4.eq.6)then
            
            seign(2) = eign(4)
            do i = 1,6
               BT(2,i) = A(i,4) 
            enddo

         endif

!------pos5

         if(slep5.eq.1)then

            seign(5) = eign(5)
            do i = 1,6
               BT(5,i) = A(i,5) 
            enddo

         elseif(slep5.eq.2)then
            
            seign(3) = eign(5)
            do i = 1,6
               BT(3,i) = A(i,5) 
            enddo

         elseif(slep5.eq.3)then
            
            seign(1) = eign(5)
            do i = 1,6
               BT(1,i) = A(i,5) 
            enddo

         elseif(slep5.eq.4)then
            
            seign(6) = eign(5)
            do i = 1,6
               BT(6,i) = A(i,5) 
            enddo

         elseif(slep5.eq.5)then
            
            seign(4) = eign(5)
            do i = 1,6
               BT(4,i) = A(i,5) 
            enddo

         elseif(slep5.eq.6)then
            
            seign(2) = eign(5)
            do i = 1,6
               BT(2,i) = A(i,5) 
            enddo

         endif

!------pos6

         if(slep6.eq.1)then

            seign(5) = eign(6)
            do i = 1,6
               BT(5,i) = A(i,6) 
            enddo

         elseif(slep6.eq.2)then
            
            seign(3) = eign(6)
            do i = 1,6
               BT(3,i) = A(i,6) 
            enddo

         elseif(slep6.eq.3)then
            
            seign(1) = eign(6)
            do i = 1,6
               BT(1,i) = A(i,6) 
            enddo

         elseif(slep6.eq.4)then
            
            seign(6) = eign(6)
            do i = 1,6
               BT(6,i) = A(i,6) 
            enddo

         elseif(slep6.eq.5)then
            
            seign(4) = eign(6)
            do i = 1,6
               BT(4,i) = A(i,6) 
            enddo

         elseif(slep6.eq.6)then
            
            seign(2) = eign(6)
            do i = 1,6
               BT(2,i) = A(i,6) 
            enddo

         endif
      endif

!------------------------------

!===============================================

      if(flip.eq.'T')then

         counter1 = 1
         counter2 = 1

         do i = 1,6

            if(i.ne.count(1).and.i.ne.count(2))then

               do j = 1,6


                  slcnew(j,counter1) = slc(j,i)

               enddo
               slegsnew(counter1) = eign(i)
               counter1 = counter1 + 1

            else

               do j = 1,6


                  slcnewconf(j,counter2) = slc(j,i)

               enddo

               slegsnewconf(counter2) = eign(i)
               counter2 = counter2 + 1

            endif
         enddo

!--   assign eig values and vectors

         do i = 1,6

            sl1c(i) = dabs(slcnew(i,1))
            sl2c(i) = dabs(slcnew(i,2))
            sl3c(i) = dabs(slcnew(i,3))
            sl4c(i) = dabs(slcnew(i,4))
            
         enddo

         slep1c = MAXLOC(sl1c, DIM = 1)
         slep2c = MAXLOC(sl2c, DIM = 1)
         slep3c = MAXLOC(sl3c, DIM = 1)
         slep4c = MAXLOC(sl4c, DIM = 1)


         if(slep1c.eq.1)then

            seign(5) = SLegsnew(1)

            do i = 1,6
               BT(5,i) = slcnew(i,1) 
            enddo

         elseif(slep1c.eq.2)then
            
            seign(3) = SLegsnew(1)

            do i = 1,6
               BT(3,i) = slcnew(i,1) 
            enddo

         elseif(slep1c.eq.3)then
            
            seign(1) = SLegsnew(1)
            do i = 1,6
               BT(1,i) = slcnew(i,1) 
            enddo

         elseif(slep1c.eq.4)then
            
            seign(6) = SLegsnew(1)
            do i = 1,6
               BT(6,i) = slcnew(i,1) 
            enddo

         elseif(slep1c.eq.5)then
            
            seign(4) = SLegsnew(1)
            do i = 1,6
               BT(4,i) = slcnew(i,1) 
            enddo

         elseif(slep1c.eq.6)then
            
            seign(2) = SLegsnew(1)
            do i = 1,6
               BT(2,i) = slcnew(i,1) 
            enddo

         endif
!------pos2

         if(slep2c.eq.1)then

            seign(5) = SLegsnew(2)
            do i = 1,6
               BT(5,i) = slcnew(i,2) 
            enddo

         elseif(slep2c.eq.2)then
            
            seign(3) = SLegsnew(2)
            do i = 1,6
               BT(3,i) = slcnew(i,2) 
            enddo

         elseif(slep2c.eq.3)then
            
            seign(1) = SLegsnew(2)
            do i = 1,6
               BT(1,i) = slcnew(i,2) 
            enddo

         elseif(slep2c.eq.4)then
            
            seign(6) = SLegsnew(2)
            do i = 1,6
               BT(6,i) = slcnew(i,2) 
            enddo

         elseif(slep2c.eq.5)then
            
            seign(4) = SLegsnew(2)
            do i = 1,6
               BT(4,i) = slcnew(i,2) 
            enddo

         elseif(slep2c.eq.6)then
            
            seign(2) = SLegsnew(2)
            do i = 1,6
               BT(2,i) = slcnew(i,2) 
            enddo

         endif


!------pos3

         if(slep3c.eq.1)then

            seign(5) = SLegsnew(3)
            do i = 1,6
               BT(5,i) = slcnew(i,3) 
            enddo

         elseif(slep3c.eq.2)then
            
            seign(3) = SLegsnew(3)
            do i = 1,6
               BT(3,i) = slcnew(i,3) 
            enddo

         elseif(slep3c.eq.3)then
            
            seign(1) = SLegsnew(3)
            do i = 1,6
               BT(1,i) = slcnew(i,3) 
            enddo

         elseif(slep3c.eq.4)then
            
            seign(6) = SLegsnew(3)
            do i = 1,6
               BT(6,i) = slcnew(i,3) 
            enddo

         elseif(slep3c.eq.5)then
            
            seign(4) = SLegsnew(3)
            do i = 1,6
               BT(4,i) = slcnew(i,3) 
            enddo

         elseif(slep3c.eq.6)then
            
            seign(2) = SLegsnew(3)
            do i = 1,6
               BT(2,i) = slcnew(i,3) 
            enddo

         endif

!------pos4

         if(slep4c.eq.1)then

            seign(5) = SLegsnew(4)
            do i = 1,6
               BT(5,i) = slcnew(i,4) 
            enddo

         elseif(slep4c.eq.2)then
            
            seign(3) = SLegsnew(4)
            do i = 1,6
               BT(3,i) = slcnew(i,4) 
            enddo

         elseif(slep4c.eq.3)then
            
            seign(1) = SLegsnew(4)
            do i = 1,6
               BT(1,i) = slcnew(i,4) 
            enddo

         elseif(slep4c.eq.4)then
            
            seign(6) = SLegsnew(4)
            do i = 1,6
               BT(6,i) = slcnew(i,4) 
            enddo

         elseif(slep4c.eq.5)then
            
            seign(4) = SLegsnew(4)
            do i = 1,6
               BT(4,i) = slcnew(i,4) 
            enddo

         elseif(slep4c.eq.6)then
            
            seign(2) = SLegsnew(4)
            do i = 1,6
               BT(2,i) = slcnew(i,4) 
            enddo

         endif

!-------------------------------

         countempty = 1
         do i = 1,6

            if(seign(i).eq.0.d0)then
               empty(countempty) = i
               countempty = countempty + 1
            endif
         enddo



         if(min(slegsnewconf(1),slegsnewconf(2)).eq.
     $        slegsnewconf(1))then

            seign(empty(1)) = min(slegsnewconf(1),slegsnewconf(2))
            seign(empty(2)) = max(slegsnewconf(1),slegsnewconf(2))


            do i = 1,6

               BT(empty(1),i) = slcnewconf(i,1) 
               BT(empty(2),i) = slcnewconf(i,2)

            enddo
            
         else

            seign(empty(1)) = min(slegsnewconf(1),slegsnewconf(2))
            seign(empty(2)) = max(slegsnewconf(1),slegsnewconf(2))


            do i = 1,6

               BT(empty(1),i) = slcnewconf(i,2) 
               BT(empty(2),i) = slcnewconf(i,1)

            enddo

         endif


      endif

!-------------------------------------
      
      return
      
      end subroutine sorteign
 
      
!----------------------------------------------------------
